;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================    

.include "manager/game.h.s"
.include "manager/menus/menu.h.s"
.include "manager/menus/controlsmenu.h.s"
.include "manager/menus/creditsmenu.h.s"
.include "manager/menus/end_game.h.s"
.include "manager/menus/win_game.h.s"
.include "manager/menus/intro.h.s"
.include "system/input.h.s"
.include "system/render.h.s"
.include "system/interruptions.h.s"
.include "system/audio.h.s"

.area _DATA
.area _CODE

;;
;; MAIN function. This is the entry point of the application.
;;    _main:: global symbol is required for correctly compiling and linking
;;
_main::
   ;; Disable firmware to prevent it from interfering with everything
   call sys_interruption_handler_override ;; Set the interruption handler
   call sys_input_keybindings_default     ;; Init keybindings to default

   ;; Init SFX
   call sys_audio_init_sfx

   restart_all:

   ;; Init menu music
   call sys_audio_init_menu_music

   restart:

   ;; Main menu
   call man_mainmenu
   ;; Select next step
   dec a 
      jp z, start_control_menu  ;; Jump to controls
   dec a 
      jp z, start_credits_menu  ;; Jump to credits

   call man_intro
   ;; Main game
   call man_game
      jr z, start_death_screen
      jr start_win_screen
jp restart

start_control_menu:
   call man_control_menu
   jp restart

start_credits_menu:
   call man_credits_menu
   jp restart

start_death_screen:
   call man_end_game
   jp restart_all

start_win_screen:
   ;; B = Paused flag
   cp b  ;; A - B
   jr c, restart_all

   call man_win_game
   jp restart_all
