;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================    

.include "bullet.h.s"
.include "entities.h.s"
.include "../templates/templates.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Shoots a Bullet                                                  ;;
;;  Shoots a bullet given the player atk pattern                    ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys IY, A, BC and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_shoot_bullet:

    ld h, 1+ent_atk_ptrn(ix)
    ld l, ent_atk_ptrn(ix)

    ;; Checks if HL is zero
    ld a, l
    cp #0
    jr nz, jump_to_shooting
    ld a, h
    cp #0
    ret z           ;; If both H and L are zero, don't shoot

    jump_to_shooting:
    jp (hl)

ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Basic attack                                                     ;;
;;  Basic attack, behold of FORTRAN                                 ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys IY, A, BC and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_player_lvl1:
    call man_bullet_player_top_center
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Advanced attack                                                  ;;
;;  Advanced attack                                                 ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys IY, A, BC and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_player_lvl2:
    call man_bullet_player_top_left_center
    call man_bullet_player_top_right_center
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SUPER attack                                                     ;;
;;  SUPER attack, DESTROY THEM ALL                                  ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys IY, A, BC and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_player_lvl3:
    call man_bullet_player_lvl1
    call man_bullet_player_top_left
    call man_bullet_player_top_right
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Enemy pew                                                        ;;
;;   Enemy pew!                                                     ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys IY, A, BC and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_enemy_basic_atk:
    call man_bullet_enemy_bottom_center
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Create a Bullet                                                  ;;
;;  Creates a bullet given an entity (the player)                   ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys A, BC and HL                                   ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_player_top_left:
    ;; Save shooter position
    ld c, ent_x_pos(ix)
    ld b, ent_y_pos(ix)
    ld hl, #ent_template_bullet

    call man_bullet_create_bullet
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Create a Bullet                                                  ;;
;;  Creates a bullet given an entity (the player)                   ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys IY, A, BC and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_player_top_left_center:
    ld a, ent_x_pos(ix) ;; Load X pos
    ld c, ent_spr_w(ix) ;; Load shooter's width
    srl c               
    add c
    ld c, a             ;; Add half width to make bullet spawn on center
    ld a, ent_y_pos(ix) ;; Load Y pos
    ld b, #4
    sub b
    ld b, a 
    ld hl, #ent_template_bullet

    ld iy, #ent_template_bullet
    ld a, c             ;; Copy X pos in A
    ld c, ent_spr_w(iy) ;; Load Bullet's width
    srl c 
    sub c               ;; Substract bullet's width from X pos
    sub c               ;; Substract bullet's width from X pos
    ld c, a
    dec c

    call man_bullet_create_bullet
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Create a Bullet                                                  ;;
;;  Creates a bullet given an entity (the player)                   ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys IY, A, BC and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_player_top_center:
    ld a, ent_x_pos(ix) ;; Load X pos
    ld c, ent_spr_w(ix) ;; Load shooter's width
    srl c               
    add c
    ld c, a             ;; Add half width to make bullet spawn on center
    ld a, ent_y_pos(ix) ;; Load Y pos
    ld b, #4
    sub b
    ld b, a 
    ld hl, #ent_template_bullet

    ld iy, #ent_template_bullet
    ld a, c             ;; Copy X pos in A
    ld c, ent_spr_w(iy) ;; Load Bullet's width
    srl c 
    sub c               ;; Substract bullet's width from X pos
    ld c, a

    call man_bullet_create_bullet
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Create a Bullet                                                  ;;
;;  Creates a bullet given an entity (the player)                   ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys IY, A, BC and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_player_top_right_center:
    ld a, ent_x_pos(ix) ;; Load X pos
    ld c, ent_spr_w(ix) ;; Load shooter's width
    srl c               
    add c
    ld c, a             ;; Add half width to make bullet spawn on center
    ld a, ent_y_pos(ix) ;; Load Y pos
    ld b, #4
    sub b
    ld b, a 
    ld hl, #ent_template_bullet

    ;; Little adjustment
    inc c

    call man_bullet_create_bullet
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Create a Bullet                                                  ;;
;;  Creates a bullet given an entity (the player)                   ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys IY, A, BC and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_player_top_right:
    ld a, ent_x_pos(ix) ;; Load X pos
    ld c, ent_spr_w(ix) ;; Load shooter's width             
    add c
    ld c, a             ;; Add width to make bullet spawn on right side
    ld b, ent_y_pos(ix) ;; Load Y pos
    ld hl, #ent_template_bullet

    ld iy, #ent_template_bullet
    ld a, c             ;; Copy X pos in A
    ld c, ent_spr_w(iy) ;; Load Bullet's width
    sub c               ;; Substract bullet's width from X pos
    ld c, a

    call man_bullet_create_bullet
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Create a Bullet                                                  ;;
;;  Creates a bullet given an entity                                ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  Destroys IY, A, BC and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_enemy_bottom_center:
    ld a, ent_x_pos(ix) ;; Load X pos
    ld c, ent_spr_w(ix) ;; Load shooter's width
    srl c               
    add c
    ld c, a             ;; Add half width to make bullet spawn on center
    ld b, ent_y_pos(ix) ;; Load Y pos
    ld a, ent_spr_h(ix) ;; Load shooter's height
    add b
    ld b, a
    ld hl, #ent_template_enemy_bullet

    ld iy, #ent_template_enemy_bullet
    ld a, c             ;; Copy X pos in A
    ld c, ent_spr_w(iy) ;; Load Bullet's width
    srl c 
    sub c               ;; Substract bullet's width from X pos
    ld c, a

    call man_bullet_create_bullet
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Create Bullet PEW PEW PEW                                        ;;
;;  Creates a basic bullet given a template and a position          ;;
;; Input: HL with template & BC with position                       ;;
;; Output:  Destroys A                                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_bullet_create_bullet:

    push ix                     ;;Save IX, save the child
    ;; Create the bullet
    push bc                     ;; Backup of the position 
    call man_create_one_entity
    pop bc                      ;; Recover the position

    ;; Check if the bullet was created (may have failed if no more entities can be created)
    or a
    jr z, bullet_creation_failed ;; Skip bullet modification

        ;; Modify bullet properties
        ld ent_x_pos(ix), c     ;; Modify X position to match shooter
        ld ent_y_pos(ix), b     ;; Modify Y position to match shooter

    bullet_creation_failed:

    pop ix                      ;; Recover the child
ret

