;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================   

.module Manager_Entities

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CONSTANTS                                                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Entity properties
ent_type        = 0
ent_comps       = 1
ent_x_pos       = 2
ent_y_pos       = 3
ent_x_vel       = 4
ent_y_vel       = 5
ent_spr_w       = 6
ent_spr_h       = 7
ent_sprite      = 8  ;; 2 bytes
ent_ai_behave   = 10 ;; 2 bytes
ent_ai_counter  = 12
ent_ai_counter2 = 13
ent_coll_mask   = 14
ent_health      = 15
ent_points      = 16
ent_anim        = 17 ;; 2 bytes
ent_anim_count  = 19
ent_death_beh   = 20 ;; 2 bytes
ent_atk_ptrn    = 22 ;; 2 bytes
ent_prev_render = 24 ;; 2 bytes
ent_prev_prev_render = 26 ;; 2 bytes
ent_prev_x      = 28
ent_prev_y      = 29
ent_prev_prev_x = 30
ent_prev_prev_y = 31

;;; Entity data
entity_size     = 32
entity_max_count = 50
entities_array_byte_size =  entity_size*entity_max_count

;;; Entity type masks
ent_type_invalid        = 0x00
ent_type_player         = 0x01
ent_type_enemy          = 0x02
ent_type_bullet         = 0x04
ent_type_enemy_bullet   = 0x08
ent_type_powerup        = 0x10
ent_type_dead           = 0x80
ent_type_default        = ent_type_enemy

;;; Entity component masks
ent_cmp_renderable  = 0x01
ent_cmp_movable     = 0x02
ent_cmp_input       = 0x04
ent_cmp_ai          = 0x08
ent_cmp_collider    = 0x10 ; 16
ent_cmp_animated    = 0x20 ; 32
ent_cmp_erase_sp    = 0x40 ; 64
ent_cmp_erase_sp2   = 0x80 ; 128
ent_cmp_default     = 0x00 ; Empty, with no components

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; ROUTINES                                                         ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.globl man_create_one_entity
.globl man_destroy_one_entity
.globl man_mark_one_entity_for_destruction
.globl man_mark_for_destroy_entity_of_type
.globl man_update_dead_entities
.globl man_for_all_matching
.globl man_for_all_pairs_matching
.globl man_search_type
.globl man_empty_entities
