;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================    

.include "controlsmenu.h.s"
.include "cpctelera.h.s"
.include "../../system/render.h.s"
.include "../../system/input.h.s"
.include "../../system/audio.h.s" 
.include "../../sprites/compressed/controlartwork.h.s"

;; Compression
.globl cpct_zx7b_decrunch_s_asm
;; Strings
.globl cpct_setDrawCharM0_asm
.globl cpct_setDrawCharM0_asm
.globl cpct_drawStringM0_asm
;; Keyboard
.globl cpct_scanKeyboard_f_asm
.globl cpct_isKeyPressed_asm

str_wasd_text:: .asciz  "WASD"
str_qopa_text:: .asciz  "OPQA"

is_switch_mode_btn_pressed: .db 00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Controls Execute                                                 ;;
;;  Entry point, runs the controls init and the control menu itself ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_control_menu:
    call man_control_menu_init
    call man_control_menu_run
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Controls Init                                                    ;;
;; Inits the control's menu                                         ;;
;; Input:                                                           ;;
;; Output:                                                          ;;
;; Destroys: HL, IY, AF, BC, DE, IX                                 ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_control_menu_init:

    ;; Black out the screen
    call sys_render_menu_init

    ld de, #0xFFFF
    ld hl, #_controlartwork_end
    call cpct_zx7b_decrunch_s_asm

    ;; Show mode string
    call sys_input_get_status           ;; Load in A the input status
    call man_show_mode_string_controls  ;; Input: A with the status
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Menu Run                                                         ;;
;; Runs the game's menu loop                                        ;;
;; Input: -                                                         ;;
;; Output:  -                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_control_menu_run:
    ;; Check mute/unmute
    call sys_audio_check_mute

    ;; Check if keybindings switch key was pressed
    call man_controls_switch_mode_controls

    ;; Check if BACK key was pressed
    ld hl, #key_back
    call cpct_isKeyPressed_asm
    jr z, man_control_menu_run ;; If not, check again

    ;; Hard screen erase
    cpctm_clearScreen_asm 0x00 ;; 393236 CPU Cycles

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Menu Switch Control Mode                                             ;;
;;  Checks mode change key and switches between WASD and OPQA           ;;
;; Input: -                                                             ;;
;; Output:  Destroys AF, BC, DE and HL                                  ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_controls_switch_mode:
    ;; Check key pressed
    call cpct_scanKeyboard_f_asm

    ld hl, #keyid_mode
    ld de, #man_controls_switch_mode_action
    ld bc, #is_switch_mode_btn_pressed
    call sys_input_check_pressed_once
ret
man_controls_switch_mode_action:
    ;; Switch bindings
    call sys_input_switch_keybindings

    ;; New press, switch modes
    string_show_routine = .+1
    call man_show_mode_string_controls
ret
man_controls_switch_mode_controls:
    ld hl, #man_show_mode_string_controls
    ld (string_show_routine), hl
    call man_controls_switch_mode
ret
man_controls_switch_mode_mainmenu:
    ld hl, #man_show_mode_string_mainmenu
    ld (string_show_routine), hl
    call man_controls_switch_mode
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Menu Draw Mode String                                                ;;
;;  Draws a string indicating the key mode we are using                 ;;
;; Input: A = 0 if QOPA mode, A = 0xFF if WASD mode                     ;;
;; Output:  Destroys IX, IY, AF, BC, DE and HL                          ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_show_mode_string_controls:
    ld de, #screen_pos_controls
    jp man_control_menu_switch_mode_string
man_show_mode_string_mainmenu:
    ld de, #screen_pos_mainmenu
    jp man_control_menu_switch_mode_string
man_control_menu_switch_mode_string:
    ;; A = #0x00 if QOPA
    ;; A = #0xFF if WASD

    or a
    jr z, string_mode_draw
        call man_menu_draw_wasd
        ret
    string_mode_draw:
        call man_menu_draw_qopa
ret

man_menu_draw_wasd:
    ld hl, #0x0009
    ld iy, #str_wasd_text
    ;; DE = Screen Memory position
    call man_menu_draw_single_string
ret
man_menu_draw_qopa:
    ld hl, #0x0009
    ld iy, #str_qopa_text
    ;; DE = Screen Memory position
    call man_menu_draw_single_string
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Menu Draw String                                                     ;;
;;  Draws a string given a color, a string and a Y coordinate           ;;
;; Input: HL with color, IY with string and DE with memory pos          ;;
;; Output:  Destroys IX, AF, BC, DE and HL                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_draw_single_string:
    ;; Save screen memory backup
    push de

    ;; Set Color
    call cpct_setDrawCharM0_asm

    pop hl  ;; HL = Screen memory pos
    ;; We paint
    call cpct_drawStringM0_asm
ret
