;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================    

.include "creditsmenu.h.s"
.include "cpctelera.h.s"
.include "menu.h.s"
.include "../../system/render.h.s"
.include "../../system/input.h.s"
.include "../../system/audio.h.s" 
.include "../../sprites/compressed/creditsartwork.h.s"

;; Compression
.globl cpct_zx7b_decrunch_s_asm
;; Keyboard
.globl cpct_scanKeyboard_f_asm
.globl cpct_isKeyPressed_asm
;; Video
.globl cpct_waitVSYNC_asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Credits Execute                                                  ;;
;;  Entry point, runs the credits init and the credits menu itself  ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_credits_menu:
    call man_credits_menu_init
    call man_credits_menu_run
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Credits Init                                                     ;;
;; Inits the credit's menu                                          ;;
;; Input:                                                           ;;
;; Output:                                                          ;;
;; Destroys: HL, IY, AF, BC, DE, IX                                 ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_credits_menu_init:

    ;; Black out the screen
    call sys_render_menu_init

    ld de, #0xFFFF
    ld hl, #_creditsartwork_end
    call cpct_zx7b_decrunch_s_asm
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Credits Run                                                      ;;
;; Runs the credit's menu loop                                      ;;
;; Input: -                                                         ;;
;; Output:  -                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_credits_menu_run:
    ;; Check mute/unmute
    call sys_audio_check_mute

    ;; Rotate palette
    call man_menu_rotate_palette
    halt
    halt
    call cpct_waitVSYNC_asm

    call cpct_scanKeyboard_f_asm

    ;; Check if BACK key was pressed
    ld hl, #key_back
    call cpct_isKeyPressed_asm
    jr z, man_credits_menu_run ;; If not, check again

    ;; Hard screen erase
    cpctm_clearScreen_asm 0x00 ;; 393236 CPU Cycles

    ;; Reset palette
    call sys_render_set_palette
ret
