;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================    

.include "end_game.h.s"
.include "cpctelera.h.s"
.include "menu.h.s" 
.include "../../manager/huditems.h.s" 
.include "../../system/score.h.s" 
.include "../../system/render.h.s" 
.include "../../sprites/compressed/gameoverartwork.h.s"

;; Compression
.globl cpct_zx7b_decrunch_s_asm
;; Keyboard
.globl cpct_scanKeyboard_f_asm
.globl cpct_isAnyKeyPressed_f_asm
;; Video
.globl cpct_waitVSYNC_asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; End Game Execute                                                 ;;
;;  Entry point, runs the end game screen's init and loop           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_end_game:
    call man_end_game_init
    call man_end_game_run
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; End Game Init                                                    ;;
;; Inits the End Game 's menu                                       ;;
;; Input:                                                           ;;
;; Output:                                                          ;;
;; Destroys: HL, IY, AF, BC, DE, IX                                 ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_end_game_init:
    ;; Black out the screen, put screen size to 20x25
    call sys_render_menu_init

    ;; Show artwork
    ld de, #0xFFFF
    ld hl, #_gameoverartwork_end
    call cpct_zx7b_decrunch_s_asm

    ;; Show score string
    call sys_score_end_game_draw    ;; Create string
    call sys_render_all_menu_hud    ;; Show string
    call man_empty_hud              ;; Delete useless hud internal data
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; End Game  Run                                                    ;;
;; Runs the End Game 's menu loop                                   ;;
;; Input: -                                                         ;;
;; Output:  Destroys AF, BC, DE and HL                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_end_game_run:
    ;; Rotate palette for fire animation
    call man_menu_rotate_palette
    halt
    halt
    call cpct_waitVSYNC_asm

    ;; If time counter = 0, back to main menu
    jp z, finish_end_screen

    ;; Making this go slower
    call wait_a_frame

    ;; Check if any key is pressed
    call cpct_scanKeyboard_f_asm
    call cpct_isAnyKeyPressed_f_asm
    jr z, man_end_game_run

        end_game_loop_release:
            call cpct_scanKeyboard_f_asm
            call cpct_isAnyKeyPressed_f_asm
        jr nz, end_game_loop_release

    finish_end_screen:

    ;; Hard screen erase
    cpctm_clearScreen_asm 0x00 ;; 393236 CPU Cycles

    ;; Reset palette
    call sys_render_set_palette
ret
