;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================    

.include "intro.h.s"
.include "cpctelera.h.s"
.include "../../system/render.h.s"
.include "../../system/audio.h.s"
.include "../../sprites/compressed/introscreen.h.s"

;; Compression
.globl cpct_zx7b_decrunch_s_asm

;; Video
.globl cpct_waitVSYNC_asm

;;Keyboard
.globl cpct_scanKeyboard_f_asm
.globl cpct_isAnyKeyPressed_f_asm

intro_counter: .db 00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Intro Execute                                                    ;;
;;  Entry point, runs the intro init and the intro itself           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_intro:
    call man_intro_init
    call man_intro_run
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Intro Init                                                       ;;
;; Inits the game's intro                                           ;;
;; Input:                                                           ;;
;; Output:                                                          ;;
;; Destroys: HL, IY, AF, BC, DE, IX                                 ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_intro_init:
    ;; Black out the screen
    call sys_render_menu_init

    ;;Init intro_counter
    ld a, #250
    ld (intro_counter),a

    ;; Render menu artwork
    ld de, #0xFFFF
    ld hl, #_introscreen_end
    call cpct_zx7b_decrunch_s_asm

    ;; Init music
    call sys_audio_init_intro_music
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Intro Run                                                        ;;
;; Runs the game's intro loop                                       ;;
;; Input: -                                                         ;;
;; Output:  -                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_intro_run:

    call cpct_scanKeyboard_f_asm
    call cpct_isAnyKeyPressed_f_asm
    jr z, man_intro_loop_run

        intro_game_loop_release:
            call cpct_scanKeyboard_f_asm
            call cpct_isAnyKeyPressed_f_asm
        jr nz, intro_game_loop_release

        jp intro_finish

    man_intro_loop_run:

    ;;Check if time ran out
    

        halt
        halt
        call cpct_waitVSYNC_asm
        halt
        halt
        call cpct_waitVSYNC_asm

    ld a, (intro_counter)

        dec a
        ld (intro_counter), a

    jp nz, man_intro_run


    intro_finish:

    ;; Hard screen erase
    cpctm_clearScreen_asm 0x00 ;; 393236 CPU Cycles

ret
