;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================    

.include "menu.h.s"
.include "controlsmenu.h.s"
.include "cpctelera.h.s"
.include "../huditems.h.s"
.include "../../system/render.h.s"
.include "../../system/input.h.s"
.include "../../system/audio.h.s"
.include "../../system/score.h.s"
.include "../../sprites/compressed/mainmenuartwork.h.s"

;; Compression
.globl cpct_zx7b_decrunch_s_asm
;; Keyboard
.globl cpct_scanKeyboard_f_asm
.globl cpct_isKeyPressed_asm
;; Video
.globl cpct_setPALColour_asm
.globl cpct_waitVSYNC_asm

palette_change:   .db 01
palette_rotation: .db 01

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Menu Execute                                                     ;;
;;  Entry point, runs the menu init and the menu itself             ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_mainmenu:
    call man_menu_init
    call man_menu_run
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Menu Init                                                        ;;
;; Inits the game's menu                                            ;;
;; Input:                                                           ;;
;; Output:                                                          ;;
;; Destroys: HL, IY, AF, BC, DE, IX                                 ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_init:
    ;; Black out the screen
    call sys_render_menu_init

    ;; Render menu artwork
    ld de, #0xFFFF
    ld hl, #_mainmenuartwork_end
    call cpct_zx7b_decrunch_s_asm

    ;; Show HiScore
    call sys_score_hiscore_mainmenu_draw
    call sys_render_all_menu_hud    ;; Show string
    call man_empty_hud              ;; Delete useless hud internal data

    ;; Show mode string
    call sys_input_get_status           ;; Load in A the input status
    call man_show_mode_string_mainmenu  ;; Input: A with the status
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Menu Run                                                         ;;
;; Runs the game's menu loop                                        ;;
;; Input: -                                                         ;;
;; Output:  -                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_run:

    ;; Rotate palette for fire animation
    call man_menu_rotate_palette
    halt
    halt
    call cpct_waitVSYNC_asm

    ;; Check mute/unmute
    call sys_audio_check_mute

    ;; Check mode change
    call man_controls_switch_mode_mainmenu

    ;; Check Keyboard
    call cpct_scanKeyboard_f_asm

        ;; Check if CONTROLS key was pressed
        ld hl, #key_controls
        call cpct_isKeyPressed_asm
        jr z, controls_skip         ;; If not, keep checking
            ld a, #1
            jp man_menu_exit
        controls_skip:

        ;; Check if CREDITS key was pressed
        ld hl, #key_credits
        call cpct_isKeyPressed_asm
        jr z, credits_skip         ;; If not, keep checking
            ld a, #2
            jp man_menu_exit
        credits_skip:

        ;; Check if PLAY key was pressed
        ld hl, #key_play
        call cpct_isKeyPressed_asm
        jr z, man_menu_run          ;; If not, check again
            ld a, #10

    man_menu_exit:
    ;; Hard screen erase
    cpctm_clearScreen_asm 0x00 ;; 393236 CPU Cycles

    ;; Reset palette
    push af
    call sys_render_set_palette
    pop af

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Rotate Palette                                                   ;;
;;  Rotates colors to make a pseudo-animation                       ;;
;; Input: -                                                         ;;
;; Output:  Destroys AF, B, D and HL                                ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_menu_rotate_palette:
    ;; Base palette
    ;;  4 -> Red
    ;;  9 -> Orange
    ;; 13 -> Yellow
    ;; Yellow -> Orange -> Red

    ;; Update the counter
    ld a, (palette_change)   ;; Load counter
    rrca                       ;; Move the bit
    ld (palette_change), a   ;; Save counter
    and #1
    ret z

    ;; Update the rotation counter
    ld a, (palette_rotation)   ;; Load counter
    rrca                       ;; Move the bit
    ld (palette_rotation), a   ;; Save counter

        ;; Save counter backup in D
        ld d, a

        ;; Select animation frame to execute
        and #0b01000000 ;; Bit 7
        jr z, skip_bit_7
            ;; BIT 7 FRAME

            ;; Set red to orange
            ld l, #4                    ;; Base red
            ld h, #0x0E                 ;; To Orange
            call cpct_setPALColour_asm  ;; Input: HL, destroys AF and B

            ;; Set orange to yellow
            ld l, #9                    ;; Base orange
            ld h, #0x0A                 ;; To Yellow
            call cpct_setPALColour_asm  ;; Input: HL, destroys AF and B

            ;; Set yellow to red
            ld l, #13                   ;; Base yellow
            ld h, #0x0C                 ;; To Red
            call cpct_setPALColour_asm  ;; Input: HL, destroys AF and B

            ret ;; End
        skip_bit_7:

        ld a, d
        and #0b00010000 ;; Bit 5
        jr z, skip_bit_5
            ;; BIT 5 FRAME

            ;; Set red to orange
            ld l, #4                    ;; Base red
            ld h, #0x0A                 ;; To Orange
            call cpct_setPALColour_asm  ;; Input: HL, destroys AF and B

            ;; Set orange to yellow
            ld l, #9                    ;; Base orange
            ld h, #0x0C                 ;; To Yellow
            call cpct_setPALColour_asm  ;; Input: HL, destroys AF and B

            ;; Set yellow to red
            ld l, #13                   ;; Base yellow
            ld h, #0x0E                 ;; To Red
            call cpct_setPALColour_asm  ;; Input: HL, destroys AF and B

            ret ;; End
        skip_bit_5:

        ld a, d
        and #0b00000100 ;; Bit 3
        jr z, skip_bit_3
            ;; BIT 3 FRAME

            ;; Set red to orange
            ld l, #4                    ;; Base red
            ld h, #0x0C                 ;; To Orange
            call cpct_setPALColour_asm  ;; Input: HL, destroys AF and B

            ;; Set orange to yellow
            ld l, #9                    ;; Base orange
            ld h, #0x0E                 ;; To Yellow
            call cpct_setPALColour_asm  ;; Input: HL, destroys AF and B

            ;; Set yellow to red
            ld l, #13                   ;; Base yellow
            ld h, #0x0A                 ;; To Red
            call cpct_setPALColour_asm  ;; Input: HL, destroys AF and B

            ld a, #01 ;; Skip frames 1 and 2
            ld (palette_rotation), a   ;; Save counter

            ret ;; End
        skip_bit_3:
ret
