;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================   

.include "animations.h.s"
.include "../manager/entities.h.s"
.include "../manager/huditems.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Update One Animation                                             ;;
;;  Updates the animation of the specified entity                   ;;
;; Input: IX with entity's memory position                          ;;
;; Output:  -                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_anim_update_one:
    ;; Decrease the animation counter
    dec ent_anim_count(ix)
    ;; If zero, load next frame
    ret nz

        ;; Go to next animation (Add [anim_size] to entity's animation)
        ld l, ent_anim(ix)  ;; Load animation pointer in HL
        ld h, 1+ent_anim(ix)    
        ld de, #anim_size   ;; Load animation size
        add hl, de          ;; Add it to animation pointer
        ld ent_anim(ix), l  ;; Save animation pointer
        ld 1+ent_anim(ix), h

        anim_frame_setup:
        ;; Make copy of animation frame in IY
        push hl             
        pop iy
        

        ;; Load next frame's time in the entity's counter
        ld a, anim_time(iy)         ;; Load time into A
        ld ent_anim_count(ix), a    ;; Save time into animation counter

        ;; If new time is zero, then reset the animation array
        or a
        jr nz, animation_reset_skip
            ;; Copy array's start into entity's animation
            ld l, anim_sprite(iy)   ;; Load animation pointer (to array start) in HL
            ld h, 1+anim_sprite(iy) ;; Sprite now is actually the beggining of the array

            ld ent_anim(ix), l      ;; Save animation pointer
            ld 1+ent_anim(ix), h

            ;; Repeat frame's setup
            jr anim_frame_setup

        animation_reset_skip:
        ;; Load next frame's sprite in the entity's sprite
        ld l, anim_sprite(iy)   ;; Load sprite ptr in HL
        ld h, 1+anim_sprite(iy)    
        ld ent_sprite(ix), l    ;; Save new sprite ptr in entity
        ld 1+ent_sprite(ix), h
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Update One Animation (HUD)                                       ;;
;;  Updates the animation of the specified hud item                 ;;
;; Input: IX with hud item's memory position                        ;;
;; Output:  -                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_anim_hud_update_one:
    ;; Check animation component
    ld a, hud_comps(ix)
    and #hud_comp_animated
    ret z

    ;; Decrease the animation counter
    dec hud_anim_count(ix)
    ;; If zero, load next frame
    ret nz

        ;; Go to next animation (Add [anim_size] to entity's animation)
        ld l, hud_anim(ix)  ;; Load animation pointer in HL
        ld h, 1+hud_anim(ix)    
        ld de, #anim_size   ;; Load animation size
        add hl, de          ;; Add it to animation pointer
        ld hud_anim(ix), l  ;; Save animation pointer
        ld 1+hud_anim(ix), h

        anim_hud_frame_setup:
        ;; Make copy of animation frame in IY
        push hl             
        pop iy

        ;; Load next frame's time in the entity's counter
        ld a, anim_time(iy)         ;; Load time into A
        ld hud_anim_count(ix), a    ;; Save time into animation counter

        ;; If new time is zero, then reset the animation array
        or a
        jr nz, animation_hud_reset_skip
            ;; Copy array's start into entity's animation
            ld l, anim_sprite(iy)   ;; Load animation pointer (to array start) in HL
            ld h, 1+anim_sprite(iy) ;; Sprite now is actually the beggining of the array

            ld hud_anim(ix), l      ;; Save animation pointer
            ld 1+hud_anim(ix), h

            ;; Repeat frame's setup
            jr anim_hud_frame_setup

        animation_hud_reset_skip:
        ;; Load next frame's sprite in the entity's sprite
        ld l, anim_sprite(iy)   ;; Load sprite ptr in HL
        ld h, 1+anim_sprite(iy)    
        ld hud_sprite(ix), l    ;; Save new sprite ptr in entity
        ld 1+hud_sprite(ix), h
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Update All Animations                                            ;;
;;  Asks the manager to update all animations                       ;;
;; Input: -                                                         ;;
;; Output:  -                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_anim_update_all:
    ld hl, #sys_anim_update_one
    ld b, #ent_cmp_animated
    call man_for_all_matching

    ld hl, #sys_anim_hud_update_one
    call man_hud_for_all
ret
