;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================    

.include "audio.h.s"
.include "input.h.s"
.include "../music/HyperSpaceMenu.h.s"
.include "../music/HyperSpaceIntro.h.s"
.include "../music/HyperSpaceSFX_Examples.h.s"
.include "cpctelera.h.s"

; Audio
.globl cpct_akp_musicInit_asm
.globl cpct_akp_SFXInit_asm
.globl cpct_akp_SFXPlay_asm
.globl cpct_akp_stop_asm
; Keyboard
.globl cpct_scanKeyboard_f_asm
.globl cpct_isKeyPressed_asm

is_muted: .db 01
is_mute_btn_pressed: .db 00

sys_audio_check_mute:
    ;; Check key pressed
    call cpct_scanKeyboard_f_asm

    ld hl, #key_mute
    ld de, #sys_audio_switch_mute_state
    ld bc, #is_mute_btn_pressed
    call sys_input_check_pressed_once
ret

sys_audio_switch_mute_state:
    ld a, (is_muted)
    or a
    jp nz, sys_audio_mute
    jp z,  sys_audio_unmute
ret

sys_audio_mute:
    ld a, #0
    ld (is_muted), a
    call cpct_akp_stop_asm      ;; Stop previous music
ret

sys_audio_unmute:
    ld a, #1
    ld (is_muted), a
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Init SFX                                                         ;;
;;  Sets the SFX of the game to be played                           ;;
;; Input: -                                                         ;;
;; Output:  Destroys AF, DE, HL                                     ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_audio_init_sfx:
    ld de, #_audio_sfx          ;; DE = Pointer to SFX Music
    call cpct_akp_SFXInit_asm   ;; Initialize instruments for SFX
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Init Menu Music                                                  ;;
;;  Sets the music of the menu to be played                         ;;
;; Input: -                                                         ;;
;; Output:  Destroys AF, AF’, BC, DE, HL, IX, IY                    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_audio_init_menu_music:
    check_mute

    ld de, #_music_menu
    call cpct_akp_musicInit_asm
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Init Game Music                                                  ;;
;;  Sets the music of the game to be played                         ;;
;; Input: -                                                         ;;
;; Output:  Destroys AF, AF’, BC, DE, HL, IX, IY                    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_audio_init_game_music:
    check_mute

    ld de, #_music_menu
    call cpct_akp_musicInit_asm
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Init Intro Music                                                 ;;
;;  Sets the music of the intro to be played                        ;;
;; Input: -                                                         ;;
;; Output:  Destroys AF, AF’, BC, DE, HL, IX, IY                    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_audio_init_intro_music:
    check_mute

    ld de, #_music_intro
    call cpct_akp_musicInit_asm
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Init Win Music                                                   ;;
;;  Sets the music of when you beat the game to be played           ;;
;; Input: -                                                         ;;
;; Output:  Destroys AF, AF’, BC, DE, HL, IX, IY                    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_audio_init_win_music:
    check_mute

    ld de, #_music_menu
    call cpct_akp_musicInit_asm
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Play SFX - Player Shoot                                          ;;
;;  Plays a sound of a player ship shooting                         ;;
;; Input: -                                                         ;;
;; Output:  Destroys AF, BC, DE and HL                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_audio_play_sfx_playershoot:
    check_mute

    ld  l, #0x01    ;; L  = Instrument     (>0)
    ld  h, #0x0F    ;; H  = Volume         (0-15)
    ld  e, #75      ;; E  = Note           (0-143)
    ld  d, #0x00    ;; D  = Speed          (0 = original, 1-255 speed (1 is faster))
    ld bc, #0xFFFF  ;; BC = Inverted Pitch (More pitch, lower sound)
    ld  a, #0x02    ;; A  = Channel Mask   (3 last bits are the mask)
    push ix
    call cpct_akp_SFXPlay_asm
    pop ix
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Play SFX - Explosion                                             ;;
;;  Plays a sound of a generic explosion                            ;;
;; Input: -                                                         ;;
;; Output:  Destroys AF, BC, DE and HL                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_audio_play_sfx_explosion:
    check_mute

    ld  l, #0x02    ;; L  = Instrument     (>0)
    ld  h, #0x0A    ;; H  = Volume         (0-15)
    ld  e, #64      ;; E  = Note           (0-143)
    ld  d, #0       ;; D  = Speed          (0 = original, 1-255 speed (1 is faster))
    ld bc, #0x0000  ;; BC = Inverted Pitch (More pitch, lower sound)
    ld  a, #0x02    ;; A  = Channel Mask   (3 last bits are the mask)
    push ix
    call cpct_akp_SFXPlay_asm
    pop ix
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Play SFX - Great Explosion                                       ;;
;;  Plays a sound of a generic but big explosion                    ;;
;; Input: -                                                         ;;
;; Output:  Destroys AF, BC, DE and HL                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_audio_play_sfx_big_explosion:
    check_mute

    ld  l, #0x02    ;; L  = Instrument     (>0)
    ld  h, #0x0F    ;; H  = Volume         (0-15)
    ld  e, #50      ;; E  = Note           (0-143)
    ld  d, #0x00    ;; D  = Speed          (0 = original, 1-255 speed (1 is faster))
    ld bc, #0x00F0  ;; BC = Inverted Pitch (More pitch, lower sound)
    ld  a, #0x01    ;; A  = Channel Mask   (3 last bits are the mask)
    push ix
    call cpct_akp_SFXPlay_asm

    ld  l, #0x02    ;; L  = Instrument     (>0)
    ld  h, #0x0F    ;; H  = Volume         (0-15)
    ld  e, #54      ;; E  = Note           (0-143)
    ld  d, #0x00    ;; D  = Speed          (0 = original, 1-255 speed (1 is faster))
    ld bc, #0x00F0  ;; BC = Inverted Pitch (More pitch, lower sound)
    ld  a, #0x04    ;; A  = Channel Mask   (3 last bits are the mask)
    call cpct_akp_SFXPlay_asm
    pop ix

    jp sys_audio_play_sfx_explosion
ret


; code1 = 0x0108
; code2 = 0x0208
; code3 = 0x0207
; code4 = 0x0107
; code5 = 0x0206
; code6 = 0x0106
; .globl cpct_isAnyKeyPressed_f_asm
; sys_audio_test_sfx_loop:
;     call cpct_scanKeyboard_f_asm

;     ;; TEST [1]
;     ld hl, #code1
;     call cpct_isKeyPressed_asm
;     jr z, .+2+9+2
;             call sys_audio_play_sfx_playershoot
;             call cpct_scanKeyboard_f_asm
;             call cpct_isAnyKeyPressed_f_asm
;         jr nz, .-6

;     ;; TEST [2]
;     ld hl, #code2
;     call cpct_isKeyPressed_asm
;     jr z, .+2+9+2
;             call sys_audio_play_sfx_explosion
;             call cpct_scanKeyboard_f_asm
;             call cpct_isAnyKeyPressed_f_asm
;         jr nz, .-6

;     ;; TEST [3]
;     ld hl, #code3
;     call cpct_isKeyPressed_asm
;     jr z, .+2+9+2
;             call sys_audio_play_sfx_big_explosion
;             call cpct_scanKeyboard_f_asm
;             call cpct_isAnyKeyPressed_f_asm
;         jr nz, .-6

;     jp sys_audio_test_sfx_loop
; ret
