;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================    

.include "input.h.s"
.include "../manager/entities.h.s"
.include "../manager/game.h.s"
.include "cpctelera.h.s"

.globl cpct_scanKeyboard_f_asm
.globl cpct_isKeyPressed_asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; VARIABLES                                                ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
keybinding_status: .db 0

keybindings::
.dw #keyid_p,       #sys_input_action_move_right
.dw #keyid_o,       #sys_input_action_move_left
.dw #keyid_a,       #sys_input_action_move_down
.dw #keyid_q,       #sys_input_action_move_up
.dw #keyid_j_right, #sys_input_action_move_right
.dw #keyid_j_left,  #sys_input_action_move_left
.dw #keyid_j_down,  #sys_input_action_move_down
.dw #keyid_j_up,    #sys_input_action_move_up
.dw #keyid_j_fire1, #man_game_shoot
.dw #keyid_space,   #man_game_shoot
.dw 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Check Input                                                      ;;
;;  Checks keyboard input for input-controlled entitities           ;;
;; Input: IX with entity's memory position                          ;;
;; Output: Destroys AF, BC, DE and HL                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_input_check:
    ;; Init IX to the keybindings array start
    ld iy, #keybindings

    ;; Scan the keyboard
    call cpct_scanKeyboard_f_asm    ;; Destroys A, D, HL and BC
    jp input_keycheck_start ;; Skip first advancement

    ;; Action return point
    input_action_return_point:

        ;; Move keybindings array
        ld bc, #4
        add iy, bc

        input_keycheck_start:

        ;; Get Keybinding to check
        ld l, keyb_key(iy)
        ld h, 1+keyb_key(iy)

        ;; Check array's end (HL = 0x0000)
        ld a, #0
        add h
        add l
        ret z

        ;; Check if [MOVE_RIGHT] key was pressed
        call cpct_isKeyPressed_asm
        jr z, input_action_return_point

            ;; Load return point
            ld de, #input_action_return_point
            push de

            ;; Get action memory
            ld l, keyb_action(iy)
            ld h, 1+keyb_action(iy)

            ;; Jump to action
            jp (hl)
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Input Actions                                                    ;;
;;  Input actions to associate them to keys                         ;;
;; Input: -                                                         ;;
;; Output: Varies, never destroy IX                                 ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_input_action_move_right:
    ld a, #0x01
    ld ent_x_vel(ix), a
ret
sys_input_action_move_left:
    ld a, #0xFF
    ld ent_x_vel(ix), a
ret
sys_input_action_move_down:
    ld a, #0x02
    ld ent_y_vel(ix), a
ret
sys_input_action_move_up:
    ld a, #0xFE
    ld ent_y_vel(ix), a
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Set Keybindings to Default                                       ;;
;;  Sets the keybindings to the default mode                        ;;
;; Input: -                                                         ;;
;; Output: Destroys HL                                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_input_keybindings_default:
    ld a, #0
    ld (keybinding_status), a

    ld ix, #keybindings
    ld bc, #4

    ld hl, #keyid_p
    ld   keyb_key(ix), l
    ld 1+keyb_key(ix), h
    add ix, bc

    ld hl, #keyid_o
    ld   keyb_key(ix), l
    ld 1+keyb_key(ix), h
    add ix, bc

    ld hl, #keyid_a
    ld   keyb_key(ix), l
    ld 1+keyb_key(ix), h
    add ix, bc

    ld hl, #keyid_q
    ld   keyb_key(ix), l
    ld 1+keyb_key(ix), h
    add ix, bc
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Set Keybindings to Alternative                                   ;;
;;  Sets the keybindings to the alternative mode                    ;;
;; Input: -                                                         ;;
;; Output: Destroys HL                                              ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_input_keybindings_alternative:
    ld a, #0xFF
    ld (keybinding_status), a

    ld ix, #keybindings
    ld bc, #4

    ld hl, #keyid_d
    ld   keyb_key(ix), l
    ld 1+keyb_key(ix), h
    add ix, bc

    ld hl, #keyid_a
    ld   keyb_key(ix), l
    ld 1+keyb_key(ix), h
    add ix, bc

    ld hl, #keyid_s
    ld   keyb_key(ix), l
    ld 1+keyb_key(ix), h
    add ix, bc

    ld hl, #keyid_w
    ld   keyb_key(ix), l
    ld 1+keyb_key(ix), h
    add ix, bc
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Switch Keybindings                                               ;;
;;  Sets keybindings to default or alternative, depending on current;;
;; Input: -                                                         ;;
;; Output: Destroys A and HL                                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_input_switch_keybindings:
    ;; Invert keybinding status
    ld a, (keybinding_status)   ;; Get the status (if we're on default or alt)
    xor #0xFF                   ;; Inverts A (from 00 to FF or viceversa)
    ld (keybinding_status), a   ;; Save status
    jr nz, use_alternative

        ;; Set to default
        call sys_input_keybindings_default
        ret ;; Finish routine

    use_alternative:

        ;; Set to alternative
        call sys_input_keybindings_alternative
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Get Input Status                                                 ;;
;;  Gets the input status to check outside input routines           ;;
;; Input: -                                                         ;;
;; Output: A = 0 if default, A = FF if alternative                  ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_input_get_status:
    ld a, (keybinding_status)
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Checks a Key is pressed once                                     ;;
;;  Does something only the first frame the specified key is pressed;;
;;  Requires external scanKeyboard                                  ;;
;; Input:   HL with key                                             ;;
;;          DE with action                                          ;;
;;          BC with data saver                                      ;;
;; Output: Destroys AF, BC, DE, HL and whatever destroys the action ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_input_check_pressed_once:
    ;; Save action and data saver
    push de
    push bc

    ;; HL = key id
    call cpct_isKeyPressed_asm
    jr z, not_pressed

        ;; Key is pressed
        ;; Recover data saver pointer
        pop hl
        pop de

        ;; Check if it was pressed in previous frame
        ld a, (hl)
        or a
        ret nz  ;; If it was already pressed, return
            ;; Save that we pressed before
            ld a, #1
            ld (hl), a

            ;; Execute action
            ex de, hl
            jp (hl) ;; Will return out of this routine

            ret
    not_pressed:

        ;; Key not pressed  
        pop hl  ;; Recover BC save
        ld a, #0
        ld (hl), a
        pop hl  ;; Delete DE save
ret
