;======================= LICENSE NOTICE =================================
;This file is part of HyperSpace, a videogame for Amstrad CPC 464.
;Copyright (C) 2020 HardBytes
;
;    HyperSpace is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    HyperSpace is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with HyperSpace.  If not, see <https://www.gnu.org/licenses/>.
;========================================================================    

.include "interruptions.h.s"
.include "audio.h.s"
.include "cpctelera.h.s"

.globl cpct_akp_musicPlay_asm

;;;;;;;;;;;;;;;
;; VARIABLES ;;
;;;;;;;;;;;;;;;
interrupt_counter:: .db 00
music_counter::     .db 00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Initialize Interruption Code                                     ;;
;;  Initializes the interuption manager to increase the interrupt   ;;
;;  counter every time it's called                                  ;;
;; Input: -                                                         ;;
;; Output: Destroys DE and HL                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_interruption_handler_override:
    di      ;; Disable interruptions while modifying interruption handler
    im 1    ;; Set interruption mode 1

    ;; C3 xx xx     jp #sys_interruption_handler 39,3A,3B
    ld de, #sys_interruption_handler
    ld hl, #0x0038
    ld (hl), #0xC3
    inc hl
    ld (hl), e ;; E = interrupt counter byte 1
    inc hl
    ld (hl), d ;; D = interrupt counter byte 2

    ei      ;; Reenable interruptions
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Increase Interruption Counter                                    ;;
;;  To be called by interruption handler, adds 1 to the counter     ;;
;; Input: -                                                         ;;
;; Output: Destroys Nothing                                         ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_interruption_handler:
    ;; Save registers
    push af

    ;; Increase interruption counter
    ld a, (interrupt_counter)
    inc a
    ld (interrupt_counter), a

    ;; Check music synchronization
    ld a, (music_counter)
    inc a
    ld (music_counter), a
    cp #music_freq  ;; 6 if 50Hz or 12 if 25Hz
    jr c, skip_music_sync ;; If carry, counter<freq

        ;; Reset counter
        ld a, #0
        ld (music_counter), a

        ;; Check mute
        ld a, (is_muted)
        or a
        jr z, skip_music_sync

        ;; Save all registers
        cpctm_push bc, de, hl
        exx
        ex af, af'
        cpctm_push af, bc, de, hl

        ;; Sync music
        call cpct_akp_musicPlay_asm ;; Destroys everything you love, except IX and IY

        ;; Recover all registers
        cpctm_pop hl, de, bc, af
        exx
        ex af, af'
        cpctm_pop hl, de, bc

    skip_music_sync:

    ;; Load registers
    pop af

    ei
reti    ;; Special ret made for interruptions. Not necessary now

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Reset Interruption Counter                                       ;;
;;  Set interruption counter to zero                                ;;
;; Input: -                                                         ;;
;; Output: Destroys A                                               ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_interruption_reset_counter:
    ld a, #0
    ld (interrupt_counter), a
ret
